clear
clc
close all
hobs = [0.7279 0.4913 0.2854 0.1249]; t = [5. 10. 20. 50.];
figure(1)
clf
plot(t,hobs,'k*'); xlabel('time (hours)'); ylabel('head (meters)')

Q=50.; r=60.;
pause

figure(2)
clf
TT=[0.2:0.02:1.2]'; SS=[-0.0005:0.0001:0.004]';
[ss,tt]=meshgrid(SS',TT');


	mh1=Q/(pi*t(1)*4.)*exp(-r^2/(t(1)*4)*ss./tt)./tt;
	mh2=Q/(pi*t(2)*4.)*exp(-r^2/(t(2)*4)*ss./tt)./tt;
	mh3=Q/(pi*t(3)*4.)*exp(-r^2/(t(3)*4)*ss./tt)./tt;
	mh4=Q/(pi*t(4)*4.)*exp(-r^2/(t(4)*4)*ss./tt)./tt;


mof=(hobs(1)-mh1).^2+(hobs(2)-mh2).^2+(hobs(3)-mh3).^2+(hobs(4)-mh4).^2;

%  define contour intervals
conts=[0.64 0.32 0.16 0.08 0.04 0.02 0.01];
%  contour plot of objective function
cc=contour(ss,tt,mof,conts,'k:');
clabel(cc); hold('on');
xlabel('S');
ylabel('T');

pause

% nonlinear LS first

S=0.001; T=1.; i=1;

for n=1:5

for k=1:4
	hcal(k)=hhead(t(k),S,T);
end

b=hobs-hcal; b = b'

for j=1:4
	[dhdS dhdT]=slugd(t(j),S,T); A(j,1:2)=[dhdS dhdT];
end

dx=inv(A'*A)*A'*b

xsave(1:4,i)=[S T sqrt(b'*b) sqrt(dx'*dx)]'

S=S+dx(1)
T=T+dx(2)

i=i+1

end

figure(3)
clf
cc=contour(ss,tt,mof,conts,'k:');
hold on;
clabel(cc);
plot(xsave(1,1:5),xsave(2,1:5),'b-.');
title('Gauss-Newton');
xlabel('S');
ylabel('T');

pause

% LM (nonlinear DLS)

S=0.001; T=1.; i=1; E=eye(2); epssq=0.1

for n=1:6

for k=1:4
	hcal(k)=hhead(t(k),S,T);
end

b=(hobs-hcal)'

for j=1:4
	[dhdS dhdT]=slugd(t(j),S,T); A(j,1:2)=[dhdS dhdT];
end

dx=inv(A'*A+epssq*E)*A'*b

msave(1:4,i)=[S T sqrt(b'*b) sqrt(dx'*dx)]'

i=i+1; 

S=S+dx(1)
T=T+dx(2)

end

figure(4)
clf
cc=contour(ss,tt,mof,conts,'k:');
hold on;
clabel(cc);
plot(msave(1,1:5),msave(2,1:5),'g-.');
title('Levenberg-Marquardt');
xlabel('S');
ylabel('T');

pause

% Newton's method

S=0.001; T=1.; i=1;

for n=1:7

for j = 1:4, hcal(j)=hhead(t(j),S,T); end;
b=(hobs-hcal)'

for j = 1:4, [dhdS dhdT]=slugd(t(j),S,T); A(j,1:2)=[dhdS dhdT]; end;
BR=zeros(2,2);
for j = 1:4, [dhdS2,dhdSdT,dhdT2] = slug2d(t(j),S,T);
BR=BR+b(j)*[dhdS2 dhdSdT; dhdSdT dhdT2]; end
BR;

dx=inv(A'*A+BR)*A'*b

% note different variable!
nsave(1:4,i)=[S T sqrt(b'*b) sqrt(dx'*dx)]';

i=i+1; S=S+dx(1)
T=T+dx(2)

end

figure(5)
clf
cc=contour(ss,tt,mof,conts,'k:');
hold on;
clabel(cc);
plot(xsave(1,1:5),xsave(2,1:5),'r-.');
title('Newton');
xlabel('S');
ylabel('T');

pause

figure(6)
clf

%  now plot all solution trajectories
plot(xsave(1,1:5),xsave(2,1:5),'b-.');
hold on;
clabel(cc);
plot(msave(1,1:5),msave(2,1:5),'g-')
plot(nsave(1,1:5),nsave(2,1:5),'r--')
cc=contour(ss,tt,mof,conts,'k:');

%legend('NLS (G-N)','LM','Newton')
